﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Player : MonoBehaviour {

    [SerializeField]
    private float AttackInterval = 0.2f;

    public float jumpPower;//ジャンプ力
    public float AttackOK;
    public float AttackCancel;//攻撃不可距離
    public float StanTime = 3;//攻撃硬直時間
    private float StanNow;//硬直時間計測
    private float ZeroPoint;//高度０保存

    private bool AttackFlag;//攻撃判定
    private bool MoveFlag = true;

    public GameObject Wave;

    public Sensor GrandSencor;//着地判定用センサー
    private bool JumpFlag = true;

    private Rigidbody RB;//キャラクター保持RigitBody

    private Animator Anim;

    public GameCondacter GC;
    public GameObject Effect;
    public ParticleSystem Pa;
    public float WaitTime = 2;
    public ParticleSystem HipDropPa;

    public bool jump{
        set{ 
            JumpFlag = value;
            if (value)
            {
                if (AttackFlag)
                {
                    HipDropPa.Play();
                }
                else
                {
                    AudioManager.Instance.PlaySE("landing");
                }
                Anim.SetBool("Jump", false);
            }
            AttackCheck();
        }
        get{ return JumpFlag; }
    }

    void JumpMove(){
        //ジャンプ処理
        //攻撃後硬直
        if (AttackFlag){
            return;
        }
        Anim.SetBool("Jump", true);
        //着地判定反転
        this.jump = false;
        //上昇
        RB.AddForce(Vector3.up * jumpPower, ForceMode.VelocityChange);
    }

    void HipDrop(){
        //攻撃処理
        //上昇中攻撃不可
        if (RB.velocity.y > AttackOK){
            return;
        }
        //一定高度以下攻撃不可
        if(transform.position.y < ZeroPoint + AttackCancel){
            return;
        }
        AttackFlag = true;
        //落下
        RB.AddForce(Vector3.down * jumpPower, ForceMode.VelocityChange);
    }

    void AttackNow(){
        //硬直処理
        //硬直解除
        if (StanNow > StanTime){
            AttackFlag = false;
            StanNow = 0;
        }
        //硬直時間計測
        else{
            StanNow += Time.deltaTime;
        }
        
    }

    void Start () {
        RB = GetComponent<Rigidbody>();
        Anim = GetComponent<Animator>();
    }

    void Update() {
        if (!this.MoveFlag)
            return;
        if (!GC.PlayerMoveFlag)
            return;

        //硬直
        if (AttackFlag){
            AttackNow();
        }
    
        //アクションボタン判定
        if (Input.GetKeyDown(KeyCode.Space)){
            //着地時
            if (this.jump){
                AudioManager.Instance.PlaySE("droproll");
                JumpMove();
            }
            //空中時
            else{
                HipDrop();
            }
        }

        //高度0取得
        if (this.jump){
            ZeroPoint = transform.position.y;
        }
    }

    public void AttackCheck(){
        if (AttackFlag)
        {
            StartCoroutine("Attack");
        }
    }

    IEnumerator Attack()
    {
        Instantiate(Wave, new Vector3(transform.position.x, 0.5f, transform.position.z), Quaternion.identity);
        AudioManager.Instance.PlaySE("playerwave");
        yield return new WaitForSeconds(AttackInterval);
        Instantiate(Wave, new Vector3(transform.position.x, 0.5f, transform.position.z), Quaternion.identity);
        AudioManager.Instance.PlaySE("playerwave");
    }

    public void Ded(){

        if (this.MoveFlag)
        {
            this.MoveFlag = false;
            StartCoroutine("ded");
        }
    }

    IEnumerator ded(){
        AudioManager.Instance.PlaySE("PlayerDamage");
        Pa.Play();
        yield return new WaitForSeconds(WaitTime);
        Effect.transform.parent = null;
        GC.GameOver();
        Destroy(gameObject);
    }

}
